@@ -13,9 +13,9 @@
     [clojure.tools.deps.alpha.util.io :refer [printerrln]])
   (:import
     ;; maven-resolver-api
-    [org.eclipse.aether RepositorySystem RepositorySystemSession]
+    [org.eclipse.aether RepositorySystem RepositorySystemSession DefaultRepositoryCache]
     [org.eclipse.aether.artifact Artifact DefaultArtifact]
-    [org.eclipse.aether.repository LocalRepository RemoteRepository RemoteRepository$Builder]
+    [org.eclipse.aether.repository LocalRepository Proxy RemoteRepository RemoteRepository$Builder]
     [org.eclipse.aether.graph Dependency Exclusion]
     [org.eclipse.aether.transfer TransferListener TransferEvent TransferResource]
 
@@ -40,10 +40,10 @@
     [org.apache.maven.repository.internal MavenRepositorySystemUtils]
 
     ;; maven-resolver-util
-    [org.eclipse.aether.util.repository AuthenticationBuilder]
+    [org.eclipse.aether.util.repository AuthenticationBuilder DefaultProxySelector DefaultMirrorSelector]
 
     ;; maven-core
-    [org.apache.maven.settings DefaultMavenSettingsBuilder]
+    [org.apache.maven.settings DefaultMavenSettingsBuilder Settings Server Mirror]
 
     ;; maven-settings-builder
     [org.apache.maven.settings.building DefaultSettingsBuilderFactory]
@@ -63,28 +63,60 @@
     (.set default-builder settings-builder)))
 
 (defn- get-settings
-  ^org.apache.maven.settings.Settings []
+  ^Settings []
   (.buildSettings
     (doto (DefaultMavenSettingsBuilder.)
       (set-settings-builder (.newInstance (DefaultSettingsBuilderFactory.))))))
 
+(defn- select-mirror
+  ^RemoteRepository [^Settings settings ^RemoteRepository repo]
+  (let [mirrors (.getMirrors settings)
+        selector (DefaultMirrorSelector.)]
+    (run! (fn [^Mirror mirror] (.add selector
+                                 (.getName mirror)
+                                 (.getUrl mirror)
+                                 (.getLayout mirror)
+                                 false
+                                 (.getMirrorOf mirror)
+                                 (.getMirrorOfLayouts mirror)))
+      mirrors)
+    (.getMirror selector repo)))
+
+(defn- select-proxy
+  ^Proxy [^Settings settings ^RemoteRepository repo]
+  (->> (.getProxies settings)
+    (keep (fn [^org.apache.maven.settings.Proxy proxy-setting]
+            (when (.isActive proxy-setting)
+              (.. (DefaultProxySelector.)
+                (add (Proxy. (.getProtocol proxy-setting)
+                             (.getHost proxy-setting)
+                             (.getPort proxy-setting)
+                             (.. (AuthenticationBuilder.)
+                               (addUsername (.getUsername proxy-setting))
+                               (addPassword (.getPassword proxy-setting))
+                               build))
+                     (.getNonProxyHosts proxy-setting))
+                (getProxy repo)))))
+    first))
+
 (defn remote-repo
   ^RemoteRepository [[^String name {:keys [url]}]]
-  (let [repository (RemoteRepository$Builder. name "default" url)
-        ^org.apache.maven.settings.Server server-setting
-        (first (filter
-                 #(.equalsIgnoreCase name
-                                     (.getId ^org.apache.maven.settings.Server %))
-                 (.getServers (get-settings))))]
-    (cond-> repository
-      server-setting
-      (.setAuthentication (-> (AuthenticationBuilder.)
-                              (.addUsername (.getUsername server-setting))
-                              (.addPassword (.getPassword server-setting))
-                              (.addPrivateKey (.getPrivateKey server-setting)
-                                              (.getPassphrase server-setting))
-                              (.build)))
-      true
+  (let [^Settings settings (get-settings)
+        builder (RemoteRepository$Builder. name "default" url)
+        maybe-repo (.build builder)
+        mirror (select-mirror settings maybe-repo)
+        proxy (select-proxy settings (or mirror maybe-repo))
+        ^Server server-setting (->> (.getServers settings) (filter #(= name (.getId ^Server %))) first)]
+    (->
+      (cond-> builder
+        mirror (.setUrl (.getUrl mirror))
+        server-setting (.setAuthentication
+                         (-> (AuthenticationBuilder.)
+                           (.addUsername (.getUsername server-setting))
+                           (.addPassword (.getPassword server-setting))
+                           (.addPrivateKey (.getPrivateKey server-setting) (.getPassphrase server-setting))
+                           (.build)))
+        proxy (.setProxy proxy))
       (.build))))
 
 ;; Local repository
@@ -141,6 +173,7 @@
         local-repo-mgr (.newLocalRepositoryManager system session (make-local-repo local-repo))]
     (.setLocalRepositoryManager session local-repo-mgr)
     (.setTransferListener session console-listener)
+    (.setCache session (DefaultRepositoryCache.))
     session))
 
 (defn exclusions->data
